@@ -85,10 +85,9 @@ def final_loan_principal(payment, principal, interest, years):
 def solve_payment(payment, loan, interest, years):
     principal = initial_loan_principal(loan, interest)
     payment = flx.aitken_secant(final_loan_principal,
-                                payment, payment+10., 1e-6, 1.,
+                                payment, payment+10., 1., 10.,
                                 args=(principal, interest, years),
-                                maxiter=200,
-                                checkroot=False)
+                                maxiter=200, checkiter=False)
     return payment
 
 @njitable(cache=True)
@@ -550,8 +549,8 @@ class TEA:
         IRR = self._IRR
         args = (self.cashflow, self._duration_array)
         IRR = flx.aitken_secant(NPV_at_IRR,
-                                IRR, 1.0001 * IRR + 1e-3,
-                                maxiter=200, args=args, checkroot=False)
+                                IRR, 1.0001 * IRR + 1e-3, xtol=1e-6, ytol=10.,
+                                maxiter=200, args=args, checkiter=False)
         self._IRR = IRR
         return IRR
     
@@ -580,8 +579,8 @@ class TEA:
         args = (NPV, coefficients, discount_factors)
         sales = self._sales or stream.price * price2cost
         sales = flx.aitken_secant(NPV_with_sales,
-                                  sales, 1.0001 * sales + 1e-3,
-                                  maxiter=200, args=args, checkroot=False)
+                                  sales, 1.0001 * sales + 1e-4, ytol=10.,
+                                  maxiter=200, args=args, checkiter=False)
         self._sales = sales
         if stream.sink:
             return stream.price - sales/price2cost
@@ -797,8 +796,8 @@ class CombinedTEA(TEA):
         args = (tuple([i.cashflow for i in self.TEAs]),
                 tuple([i._duration_array for i in self.TEAs]))
         self._IRR = flx.aitken_secant(sum_NPV_at_IRR,
-                                      IRR, 1.0001 * IRR + 1e-3,
-                                      maxiter=200, args=args, checkroot=False)
+                                      IRR, 1.0001 * IRR + 1e-3, xtol=1e-6, ytol=10.,
+                                      maxiter=200, args=args, checkiter=False)
         return self._IRR
     
     def solve_price(self, stream, TEA=None):
@@ -828,8 +827,8 @@ class CombinedTEA(TEA):
         args = (NPV, coefficients, discount_factors)
         sales = self._sales or stream.price * price2cost
         sales = flx.aitken_secant(NPV_with_sales,
-                                  sales, 1.0001 * sales + 1e-3,
-                                  maxiter=200, args=args, checkroot=False)
+                                  sales, 1.0001 * sales + 1e-4, ytol=10.,
+                                  maxiter=200, args=args, checkiter=False)
         self._sales = sales
         if stream.sink:
             return stream.price - sales/price2cost
