@@ -33,6 +33,13 @@ class Variable:
     def element_name(self):
         return element_name(self.element)
     
+    @property
+    def name_with_units(self):
+        units = self.units
+        name = self.name
+        if units: name += f" [{units}]"
+        return name
+    
     @property
     def index(self):
         name = self.name
