@@ -10,12 +10,18 @@
 from . import heat_exchange
 from . import bounded_numerical_specification
 from . import stream_mass_balance
+from . import unit_group
+from . import utils
 
 __all__ = (*heat_exchange.__all__,
            *bounded_numerical_specification.__all__,
            *stream_mass_balance.__all__,
+           *unit_group.__all__,
+           *utils.__all__
 )
 
 from .heat_exchange import *
 from .bounded_numerical_specification import *
-from .stream_mass_balance import *
\ No newline at end of file
+from .stream_mass_balance import *
+from .unit_group import *
+from .utils import *
\ No newline at end of file
