@@ -1,14 +1,18 @@
 # -*- coding: utf-8 -*-
+# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
+# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# 
+# This module is under the UIUC open-source license. See 
+# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
+# for license details.
 """
-Created on Thu Aug 23 22:18:16 2018
-
-@author: yoelr
 """
-
 from ._splitter import Splitter
-from .._exceptions import DesignError
+from ..exceptions import DesignError
 from .decorators import cost
 
+__all__ = ('Clarifier',)
+
 _iswithin = lambda x, bounds: bounds[0] < x < bounds[1]
 # Electricity: 16 hp / 200 ft diameter
 
