@@ -1,19 +1,27 @@
 # -*- coding: utf-8 -*-
+# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
+# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# 
+# This module is under the UIUC open-source license. See 
+# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
+# for license details.
 """
-Created on Thu Aug 23 15:53:14 2018
-
-@author: yoelr
 """
 import numpy as np
-from fluids.pump import nema_sizes_hp
-from .design_tools.mechanical import calculate_NPSH, pump_efficiency, nearest_NEMA_motor_size
+from .design_tools.mechanical import (calculate_NPSH,
+                                      pump_efficiency, 
+                                      nearest_NEMA_motor_size,
+                                      nema_sizes_hp)
 from .design_tools.specification_factors import (
     pump_material_factors,
     pump_centrifugal_factors)
 from .._unit import Unit
 from ..utils import static_flow_and_phase
+from math import ceil
 import biosteam as bst
 
+__all__ = ('Pump',)
+
 ln = np.log
 exp = np.exp
 
@@ -40,13 +48,13 @@ class Pump(Unit):
     P=101325 : float, optional
         Pressure of output stream (Pa). If the pressure of the outlet is 
         the same as the inlet, pump is design to increase of pressure
-        to `P_design`.
+        to `dP_design`.
     pump_type='Default' : str
         If 'Default', a pump type is selected based on heuristics.
     material : str, optional
         Contruction material of pump. Defaults to 'Cast iron'.
-    P_design=405300 : float
-        Design pressure for pump.
+    dP_design=405300 : float
+        Design pressure change for pump.
     ignore_NPSH=True : bool
         Whether to take into consideration NPSH in the selection of a
         pump type.
@@ -59,10 +67,9 @@ class Pump(Unit):
     --------
     Simulate Pump for pressure increase:
     
-    >>> from thermosteam import Chemicals, Stream, settings
+    >>> from biosteam import Stream, settings
     >>> from biosteam.units import Pump
-    >>> chemicals = Chemicals(['Water', 'Ethanol'])
-    >>> settings.set_thermo(chemicals)
+    >>> settings.set_thermo(['Water', 'Ethanol'])
     >>> feed = Stream('feed', Water=200, T=350)
     >>> P1 = Pump('P1', ins=feed, outs='out', P=2e5)
     >>> P1.simulate()
@@ -76,6 +83,22 @@ class Pump(Unit):
     [0] out
         phase: 'l', T: 350 K, P: 200000 Pa
         flow (kmol/hr): Water  200
+    >>> P1.results()
+    Pump                               Units           P1
+    Power               Rate              kW        0.289
+                        Cost          USD/hr       0.0226
+    Design              Ideal power       hp        0.136
+                        Flow rate        gpm         16.3
+                        Efficiency                  0.352
+                        Actual power                0.387
+                        Pump power                    0.5
+                        N                               1
+                        Head              ft         96.5
+                        Type                  Centrifugal
+    Purchase cost       Pump             USD     4.37e+03
+                        Motor            USD          311
+    Total purchase cost                  USD     4.68e+03
+    Utility cost                      USD/hr       0.0226
     
     References
     ----------
@@ -89,7 +112,8 @@ class Pump(Unit):
               'Head': 'ft',
               'NPSH': 'ft',
               'Flow rate': 'gpm'}
-    BM = 3.3
+    _BM = {'Pump': 3.3,
+           'Motor': 3.3}
 
     @property
     def pump_type(self):
@@ -114,16 +138,17 @@ class Pump(Unit):
                             f"{', '.join(pump_material_factors)}")
         self._F_Mstr = material  
         
-    def __init__(self, ID='', ins=None, outs=(), P=101325,
+    def __init__(self, ID='', ins=None, outs=(), thermo=None, *,
+                 P=101325,
                  pump_type='Default',
                  material='Cast iron',
-                 P_design=405300,
+                 dP_design=405300,
                  ignore_NPSH=True):
-        Unit.__init__(self, ID, ins, outs)
+        Unit.__init__(self, ID, ins, outs, thermo)
         self.P = P
         self.pump_type = pump_type
         self.material = material
-        self.P_design = P_design
+        self.dP_design = dP_design
         self.ignore_NPSH = ignore_NPSH
     
     def _setup(self):
@@ -146,30 +171,33 @@ class Pump(Unit):
         mass = si.F_mass
         nu = si.nu
         dP = Po - Pi
-        if dP < 1: dP = self.P_design - Pi
-        Design['Ideal power'] = power_ideal = Qi*dP*3.725e-7 # hp
-        Design['Flow rate'] = q = Qi*4.403 # gpm
-        if power_ideal <= max_hp:
-            Design['Efficiency'] = efficiency = pump_efficiency(q, power_ideal)
-            Design['Actual power'] = power = power_ideal/efficiency
-            Design['Pump power'] = nearest_NEMA_motor_size(power)
-            Design['N'] = N = 1
-            Design['Head'] = head = power/mass*897806 # ft
-            # Note that:
-            # head = power / (mass * gravity)
-            # head [ft] = power[hp]/mass[kg/hr]/9.81[m/s^2] * conversion_factor
-            # and 897806 = (conversion_factor/9.81)
-        else:
-            power_ideal /= 2
-            q /= 2
-            if power_ideal <= max_hp:
-                Design['Efficiency'] = efficiency = pump_efficiency(q, power_ideal)
-                Design['Actual power'] = power = power_ideal/efficiency
-                Design['Pump power'] = nearest_NEMA_motor_size(power)
-                Design['N'] = N = 2
-                Design['Head'] = head = power/mass*897806 # ft
-            else:
-                raise NotImplementedError('more than 2 pump required, but not yet implemented')
+        if dP < 1: dP = self.dP_design
+        power_ideal = Qi*dP*3.725e-7 # hp
+        q = Qi*4.403 # gpm
+        # Assume 100% efficiency to estimate the number of pumps
+        # Note: i subscript refers to an individual pump
+        N_min = ceil(q / 5000)
+        N_guess = -1
+        N = max(ceil(power_ideal / max_hp), N_min)
+        while N != N_guess: 
+            N_guess = N
+            power_ideal_i = power_ideal / N_guess
+            q_i = q / N_guess
+            # Recalculate number of pumps given true efficiency
+            efficiency = pump_efficiency(q_i, power_ideal_i)
+            power = power_ideal / efficiency
+            N = max(ceil(power / max_hp), N_min)
+        Design['Ideal power'] = power_ideal_i
+        Design['Flow rate'] = q_i
+        Design['Efficiency'] = efficiency = pump_efficiency(q_i, power_ideal_i) 
+        Design['Actual power'] = power_i = power_ideal_i/efficiency
+        Design['Pump power'] = nearest_NEMA_motor_size(power_i)
+        Design['N'] = N
+        Design['Head'] = head = power / mass*897806 # ft
+        # Note that:
+        # head = power / (mass * gravity)
+        # head [ft] = power[hp]/mass[kg/hr]/9.81[m/s^2] * conversion_factor
+        # and 897806 = (conversion_factor/9.81)
         
         if self.ignore_NPSH:
             NPSH_satisfied = True
@@ -180,23 +208,27 @@ class Pump(Unit):
         # Get type
         pump_type = self.pump_type
         if pump_type == 'Default':
-            if (0.00278 < q < 5000
-                and 15.24 < head < 3200
-                and nu < 0.00002
+            if (0.00278 <= q_i <= 5000
+                and 15.24 <= head <= 3200
+                and nu <= 0.00002
                 and NPSH_satisfied):
                 pump_type = 'Centrifugal'
-            elif (q < 1500
-                  and head < 914.4
-                  and 0.00001 < nu < 0.252):
+            elif (q_i <= 1500
+                  and head <= 914.4
+                  and 0.00001 <= nu <= 0.252):
                 pump_type = 'Gear'
-            elif (head < 20000
-                  and q < 500
-                  and power < 200
-                  and nu < 0.01):
+            elif (head <= 20000
+                  and q_i <= 500
+                  and power <= 200
+                  and nu <= 0.01):
                 pump_type = 'MeteringPlunger'
             else:
                 NPSH = calculate_NPSH(Pi, si.P_vapor, si.rho)
-                raise NotImplementedError(f'no pump type available at current power ({power:.3g} hp), flow rate ({q:.3g} gpm), and head ({head:.3g} ft), kinematic viscosity ({nu:.3g} m2/s), and NPSH ({NPSH:.3g} ft)')
+                raise NotImplementedError(
+                    f'no pump type available at current power '
+                    f'({power:.3g} hp), flow rate ({q_i:.3g} gpm), and head '
+                    f'({head:.3g} ft), kinematic viscosity ({nu:.3g} m2/s), '
+                    f'and NPSH ({NPSH:.3g} ft)')
                 
         Design['Type'] = pump_type
         self.power_utility(power/N/1.341) # Set power in kW
@@ -235,8 +267,19 @@ class Pump(Unit):
                 F_T = F_Tdict['2HSC3600']
             elif p < 1450 and 100 <= q <= 1500 and 650 <= h <= 3200:
                 F_T = F_Tdict['2+HSC3600']
+            elif p < 1450 and 1500 <= q <= 5000 and 650 <= h <= 3200:
+                # TODO: This flow rate is allowable in the design section
+                # but not in the estimation of purchase cost.
+                # For now, we ignore this problem, but additional code on
+                # selecting number of pumps should be added.
+                F_T = F_Tdict['2+HSC3600']
             else:
-                raise NotImplementedError(f'no centrifugal pump available at current power ({p:.3g} hp), flow rate ({q:.3g} gpm), and head ({h:.3g} ft)')
+                # TODO: This flow rate is allowable in the design section
+                # but not in the estimation of purchase cost.
+                # For now, we ignore this problem, but additional code on
+                # selecting number of pumps should be added.
+                F_T = F_Tdict['2+HSC3600']
+                # raise NotImplementedError(f'no centrifugal pump available at current power ({p:.3g} hp), flow rate ({q:.3g} gpm), and head ({h:.3g} ft)')
             S = q*h**0.5 # Size factor
             S_new = S if S > 400 else 400
             lnS = ln(S_new)
