@@ -1,11 +1,28 @@
 # -*- coding: utf-8 -*-
+# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
+# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# 
+# This module is under the UIUC open-source license. See 
+# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
+# for license details.
 """
-Created on Thu Feb 13 09:45:49 2020
+General functional algorithms for the design and purchase cost estimation
+of distillation columns.
+
+References
+----------
+.. [1] Seider, W. D., Lewin,  D. R., Seader, J. D., Widagdo, S., Gani, R.,
+    & Ng, M. K. (2017). Product and Process Design Principles. Wiley.
+    Cost Accounting and Capital Cost Estimation (Chapter 16)
+.. [2] M. Duss, R. Taylor. (2018)
+    Predict Distillation Tray Efficiency. AICHE 
+.. [3] Green, D. W. Distillation. In Perry’s Chemical Engineers’
+    674 Handbook, 9 ed.; McGraw-Hill Education, 2018.
 
-@author: yoelr
 """
 import numpy as np
-from ...utils import approx2step
+from . import utils
+from flexsolve import njitable
 import biosteam as bst
 
 __all__ = ('compute_purchase_cost_of_trays',
@@ -24,13 +41,14 @@ __all__ = ('compute_purchase_cost_of_trays',
            'compute_tower_diameter',
            'compute_tower_height')
 
-# Minimum thickness data
-x = np.array((4, 6, 8, 10, 12)) 
-y = np.array((1/4, 5/16, 3/8, 7/16, 1/2))
-ts_min_p = np.polyfit(x,y,1)
+@njitable(cache=True)
+def minimum_thickness_from_diameter(D):
+    return 0.03125 * D + 0.125
 
+@njitable(cache=True)
 def compute_purchase_cost_of_trays(N_T, Di, F_TT, F_TM):
-    """Return total cost of all trays at BioSTEAM's CEPCI.
+    """
+    Return total cost of all trays at BioSTEAM's CEPCI.
     
     Parameters
     ----------
@@ -43,12 +61,19 @@ def compute_purchase_cost_of_trays(N_T, Di, F_TT, F_TM):
     F_TM : float
         Tray material factor.
     
+    Notes
+    -----
+    The purchase cost is given by [1]_. See source code for details.
+    The purchase cost is scaled according to BioSTEAM's Chemical
+    Plant Cost Index, `biosteam.CE`.
+    
     """
     F_CE = bst.CE/500
     C_BT = compute_tray_base_purchase_cost(Di)
     F_NT = compute_n_trays_factor(N_T)
     return N_T * F_CE * F_NT * F_TT * F_TM * C_BT
 
+@njitable(cache=True)
 def compute_purchase_cost_of_tower(Di, L, W, F_VM):
     """
     Return cost of tower at BioSTEAM's CEPCI.
@@ -62,7 +87,13 @@ def compute_purchase_cost_of_tower(Di, L, W, F_VM):
     W : float
         weight [lb].
     F_VM : float
-        Column material factor.
+        Tower material factor.
+    
+    Notes
+    -----
+    The purchase cost is given by [1]_. See source code for details.
+    The purchase cost is scaled according to BioSTEAM's Chemical
+    Plant Cost Index, `biosteam.CE`.
     
     """
     F_CE = bst.CE/500
@@ -70,19 +101,28 @@ def compute_purchase_cost_of_tower(Di, L, W, F_VM):
     C_PL = compute_plaform_ladder_cost(Di, L)
     return F_CE * (F_VM * C_V + C_PL)
 
+@njitable(cache=True)
 def compute_empty_tower_cost(W):
-    """Return the cost [C_V; in USD] of an empty tower vessel assuming a CE of 500.
+    """
+    Return the cost [C_V; in USD] of an empty tower vessel assuming a CE of 500.
     
     Parameters
     ----------
     W : float
         Weight [lb].
     
+    
+    Notes
+    -----
+    The purchase cost is given by [1]_. See source code for details.
+    
     """
     return np.exp(7.2756 + 0.18255*np.log(W) + 0.02297*np.log(W)**2)
 
+@njitable(cache=True)
 def compute_plaform_ladder_cost(Di, L):
-    """Return the cost [C_PL; in USD] of platforms and ladders assuming a CE of 500.
+    """
+    Return the cost [C_PL; in USD] of platforms and ladders assuming a CE of 500.
     
     Parameters
     ----------
@@ -91,16 +131,22 @@ def compute_plaform_ladder_cost(Di, L):
     L: float
         Legnth [ft].
     
+    Notes
+    -----
+    The purchase cost is given by [1]_. See source code for details.
+    
     """
     return 300.9*Di**0.63316*L**0.80161
 
+@njitable(cache=True)
 def compute_tower_weight(Di, L, tv, rho_M):
-    """Return the weight [W; in lb] of the tower assuming 2:1 elliptical head.
+    """
+    Return the weight [W; in lb] of the tower assuming 2:1 elliptical head.
     
     Parameters
     ----------
     Di : float
-        Inner diameter ([ft].
+        Inner diameter [ft].
     L :  float
         Legnth [ft].
     tv : float
@@ -108,11 +154,16 @@ def compute_tower_weight(Di, L, tv, rho_M):
     rho_M: floa
         Density of material [lb/in^3].
     
+    Notes
+    -----
+    The tower weight is given by [1]_. See source code for details.
+    
     """
     Di = Di*12
     L = L*12
     return np.pi*(Di+tv)*(L+0.8*Di)*tv*rho_M
 
+@njitable(cache=True)
 def compute_tower_wall_thickness(Po, Di, L, S=15000, E=None, M=29.5):
     """
     Return the wall thinkness [tv; in inches] designed to withstand the
@@ -121,18 +172,22 @@ def compute_tower_wall_thickness(Po, Di, L, S=15000, E=None, M=29.5):
     Parameters
     ----------
     Po : float
-        Operating internal pressure (psi)
+        Operating internal pressure [psi].
     Di : float
-        Internal diameter (ft)
+        Internal diameter [ft].
     L : float
-        Height (ft)
+        Height [ft].
     S : float
-        Maximum stress (psi)
+        Maximum stress [psi].
     E : float
         Fractional weld efficiency
     M : float
-        Elasticity (psi)
-        
+        Elasticity [psi].
+    
+    Notes
+    -----
+    The wall thickness is given by [1]_. See source code for details.
+    
     """
     # TODO: Incorporate temperature for choosing S and M
     Di = Di*12 # ft to in
@@ -173,24 +228,25 @@ def compute_tower_wall_thickness(Po, Di, L, S=15000, E=None, M=29.5):
     
     # Minimum thickness for vessel rigidity may be larger
     Di_ft = Di/12
-    ts_min = np.polyval(ts_min_p, Di/12) if Di_ft < 4 else 0.25
+    ts_min = minimum_thickness_from_diameter(Di_ft) if Di_ft > 4 else 0.25
     if ts < ts_min:
         ts = ts_min
     
     # Calculate thickness to withstand wind/earthquake load
     Do = Di + ts
     tw = 0.22*(Do + 18)*L**2/(S*Do**2)
-    tv = max(tw, ts)
+    tv = tw if tw > ts else ts
     
     # Vessels are fabricated from metal plates with small increments
     if tv < 0.5:
-        tv = approx2step(tv, 3/16, 1/16)
+        tv = utils.approx2step(tv, 3/16, 1/16)
     elif tv < 2:
-        tv = approx2step(tv, 0.5, 1/8)
+        tv = utils.approx2step(tv, 0.5, 1/8)
     elif tv < 3:
-        tv = approx2step(tv, 2, 1/4)
+        tv = utils.approx2step(tv, 2, 1/4)
     return tv
 
+@njitable(cache=True)
 def compute_tray_base_purchase_cost(Di):
     """Return the base cost of a tray [C_BT; USD] at a CE of 500.
     
@@ -199,9 +255,14 @@ def compute_tray_base_purchase_cost(Di):
     Di : float
         Inner diameter [ft].
     
+    Notes
+    -----
+    The purchase cost is given by [1]_. See source code for details.
+    
     """
     return 412.6985 * np.exp(0.1482*Di)
 
+@njitable(cache=True)
 def compute_n_trays_factor(N_T):
     """
     Return the cost factor for number of trays, F_NT.
@@ -209,7 +270,11 @@ def compute_n_trays_factor(N_T):
     Parameters
     ----------
     N_T: Number of trays
-        
+    
+    Notes
+    -----
+    The cost factor is given by [1]_. See source code for details.
+    
     """
     if N_T < 20:
         F_NT = 2.25/1.0414**N_T
@@ -217,6 +282,7 @@ def compute_n_trays_factor(N_T):
         F_NT = 1
     return F_NT
 
+@njitable(cache=True)
 def compute_murphree_stage_efficiency(mu, alpha, L, V):
     """
     Return the sectional murphree efficiency, E_mv.
@@ -224,13 +290,17 @@ def compute_murphree_stage_efficiency(mu, alpha, L, V):
     Parameters
     ----------
     mu: float
-        Viscosity (mPa*s)
+        Viscosity [mPa*s]
     alpha: float
         Relative volatility.    
     L: float
         Liquid flow rate by mol.
     V: float
-        Vapor flow rate by mol/
+        Vapor flow rate by mol.
+    
+    Notes
+    -----
+    The efficiency is given by [2]_. See source code for details.
     
     """
     S = alpha*V/L # Stripping factor
@@ -238,6 +308,7 @@ def compute_murphree_stage_efficiency(mu, alpha, L, V):
     if e < 1: return e
     else: return 1
 
+@njitable(cache=True)
 def compute_flow_parameter(L, V, rho_V, rho_L):
     """
     Return the flow parameter, F_LV.
@@ -253,9 +324,14 @@ def compute_flow_parameter(L, V, rho_V, rho_L):
     rho_L : float
         Liquid density.
     
+    Notes
+    -----
+    The flow parameter is given by [3]_. See source code for details.
+    
     """
     return L/V*(rho_V/rho_L)**0.5
 
+@njitable(cache=True)
 def compute_max_capacity_parameter(TS, F_LV):
     """Return the maximum capacity parameter before flooding [C_sbf; in m/s].
     
@@ -266,9 +342,14 @@ def compute_max_capacity_parameter(TS, F_LV):
     F_LV : float
         Flow parameter.
     
+    Notes
+    -----
+    The max capacity parameter is given by [3]_. See source code for details.
+    
     """
     return 0.0105 + 8.127e-4*TS**0.755*np.exp(-1.463*F_LV**0.842)
 
+@njitable(cache=True)
 def compute_max_vapor_velocity(C_sbf, sigma, rho_L, rho_V, F_F, A_ha):
     """
     Return the maximum allowable vapor velocity
@@ -289,6 +370,10 @@ def compute_max_vapor_velocity(C_sbf, sigma, rho_L, rho_V, F_F, A_ha):
     A_ha : 
         Ratio of open area, A_h, to active area, A_a
     
+    Notes
+    -----
+    The max vapor velocity is given by [3]_. See source code for details.
+    
     """
     F_ST = (sigma/20)**0.2 # Surface tension factor
     
@@ -298,10 +383,11 @@ def compute_max_vapor_velocity(C_sbf, sigma, rho_L, rho_V, F_F, A_ha):
     elif A_ha >= 0.06:
         F_HA = 5*A_ha + 0.5
     else:
-        raise ValueError(f"ratio of open to active area, 'A', must be between 0.06 and 1 ({A_ha} given)") 
+        raise ValueError("ratio of open to active area, 'A', must be between 0.06 and 1") 
     
     return C_sbf * F_HA * F_ST * ((rho_L-rho_V)/rho_V)**0.5
 
+@njitable(cache=True)
 def compute_downcomer_area_fraction(F_LV):
     """
     Return the ratio of downcomer area to net (total) area, `A_dn`.
@@ -311,6 +397,10 @@ def compute_downcomer_area_fraction(F_LV):
     F_LV : float
         Flow parameter.
 
+    Notes
+    -----
+    The fraction of downcomer area is given by [3]_. See source code for details.
+
     """
     if F_LV < 0.1:
         A_dn = 0.1
@@ -320,8 +410,9 @@ def compute_downcomer_area_fraction(F_LV):
         A_dn = 0.2
     return A_dn
 
+@njitable(cache=True)
 def compute_tower_diameter(V_vol, U_f, f, A_dn):
-    """Return column diameter [D_T; in ft].
+    """Return tower diameter [D_T; in meter].
     
     Parameters
     ----------
@@ -334,17 +425,21 @@ def compute_tower_diameter(V_vol, U_f, f, A_dn):
     A_dn : float
         Ratio of downcomer area to net (total) area.
     
+    Notes
+    -----
+    The tower diameter is given by [3]_. See source code for details.
+    
     """
     Di = (4*V_vol/(f*U_f*np.pi*(1-A_dn)))**0.5
     if Di < 0.914:
         # Make sure diameter is not too small
         Di = 0.914
-    Di *= 3.28
     return Di
 
+@njitable(cache=True)
 def compute_tower_height(TS, N_stages: int, top=True, bot=True):
     """
-    Return the height of a column [H; ft].
+    Return the height of a tower [H; in meter].
     
     Parameters
     ----------
@@ -353,6 +448,10 @@ def compute_tower_height(TS, N_stages: int, top=True, bot=True):
     N_stages : float
         Number of stages.
     
+    Notes
+    -----
+    The tower height is given by [3]_. See source code for details.
+    
     """
     # 3 m bottoms surge capacity, 1.25 m above top tray to remove entrained liquid
     H = TS*N_stages/1000
@@ -360,5 +459,4 @@ def compute_tower_height(TS, N_stages: int, top=True, bot=True):
         H += 1.2672
     if bot:
         H += 3
-    H *= 3.28
     return H 
\ No newline at end of file
