@@ -1,16 +1,24 @@
 # -*- coding: utf-8 -*-
 """
-Created on Fri Feb 22 17:31:50 2019
+Functional algorithms for the design and purchase cost estimation of
+vacuum systems.
+
+References
+----------
+.. [1] Seider, W. D.; Lewin, D. R.; Seader, J. D.; Widagdo, S.; Gani, R.;
+    Ng, M. K. Cost Accounting and Capital Cost Estimation.
+    In Product and Process Design Principles; Wiley, 2017; pp 426–485.
+
 
-@author: yoelr
 """
-from math import log as ln
-from .mechanical import motor_efficiency
+from numpy import log as ln
+from . import mechanical as mch
 from biosteam.utils import checkbounds
-from biosteam._exceptions import DesignError
+from biosteam.exceptions import DesignError
+from flexsolve import njitable
 import biosteam as bst
 
-__all__ = ('calculate_vacuum_system_power_and_cost',)
+__all__ = ('compute_vacuum_system_power_and_cost',)
 
 
 # %% Data
@@ -38,10 +46,11 @@ _air_density = 1.2041 # kg/m3 dry air
 
 # %% Calculate vacuum system requirements
 
-def calculate_vacuum_system_power_and_cost(
+def compute_vacuum_system_power_and_cost(
         F_mass, F_vol, P_suction, vessel_volume,
         vacuum_system_preference=None):
-    """Return power (kW) and cost (USD) of vacuum system.
+    """
+    Return power (kW) and cost (USD) of vacuum system.
     
     Parameters
     ----------
@@ -101,7 +110,8 @@ def get_prefered_vacuum_systems(preference):
 
 
 def get_available_vacuum_systems(F_vol_cfm, P_suction):
-    """Return available vacuum type and grade
+    """
+    Return available vacuum type and grade
     
     Parameters
     ----------
@@ -119,7 +129,8 @@ def get_available_vacuum_systems(F_vol_cfm, P_suction):
     return types
 
 def select_vacuum_system(vacuum_systems, F_vol_cfm, P_suction):
-    """Return a heuristic vacuum type and grade
+    """
+    Return a heuristic vacuum type and grade
     
     Parameters
     ----------
@@ -135,8 +146,10 @@ def select_vacuum_system(vacuum_systems, F_vol_cfm, P_suction):
                 return (vacuum_sys, grade)
     raise DesignError('no vacuum system available at current flow and suction pressure')
 
+@njitable(cache=True)
 def calculate_heuristic_air_inleakage(V, P):
-    """Return air in-leakage in kg/hr through a heuristic calculation.
+    """
+    Return air in-leakage in kg/hr through a heuristic calculation.
     
     Parameters
     ----------
@@ -144,6 +157,7 @@ def calculate_heuristic_air_inleakage(V, P):
         Vacuum volume in m3
     P : float
         Suction pressure in Pa
+    
     """
     if P > 11999.013: k = 0.2
     elif P > 4132.993: k = 0.15
@@ -152,8 +166,10 @@ def calculate_heuristic_air_inleakage(V, P):
     else: raise ValueError('cannot calculate air inleakage at pressures lower than 133.32 Pascal')
     return k*V**0.667
 
+@njitable(cache=True)
 def calculate_air_inleakage(V, P):
-    """Return air in-leakage in kg/hr.
+    """
+    Return air in-leakage in kg/hr.
     
     Parameters
     ----------
@@ -163,9 +179,10 @@ def calculate_air_inleakage(V, P):
         Suction pressure in Torr
     """
     return 5 + (0.0298 + 0.03088*ln(P) - 5.733e-4*ln(P)**2)*V**0.66
-    
+
 def calculate_vacuum_power(F_mass,  P_suction):
-    """Return vacuum power (after accounting for motor efficiency) in kW.
+    """
+    Return vacuum power (after accounting for motor efficiency) in kW.
     
     Parameters
     ----------
@@ -173,12 +190,13 @@ def calculate_vacuum_power(F_mass,  P_suction):
         Total mass flow rate entering vacuum system in kg/hr (including inleakage).
     P_suction : float
         Suction pressure in Torr
+    
     """
     SF = F_mass/P_suction # Size factor
     if SF < 0.2: SF = 0.2
     elif SF > 16: SF = 16
     Pb = 21.4*SF**0.924 # Break power assuming Liquid-ring (NASH)
-    mu_m = motor_efficiency(Pb)
+    mu_m = mch.motor_efficiency(Pb)
     return Pb/mu_m
 
 def calculate_vacuum_cost(vacuum_sys, grade, F_mass_lbph, F_vol_cfm, P_suction):
